/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef XUNFEISPEECHENGINE_P_H
#define XUNFEISPEECHENGINE_P_H

#include <kylin-ai/ai-engine/large-model/cloudspeechengine.h>
#include <thread>
#include <cpr/cpr.h>
#include <curl/curl.h>
#include <jsoncpp/json/json.h>

class XunfeiSpeechEnginePrivate : public ai_engine::lm::speech::AbstractCloudSpeechEngine
{
public:
    XunfeiSpeechEnginePrivate();
    ~XunfeiSpeechEnginePrivate();
    std::string backendName() override;

    void setConfig(const std::string &config) override;

    void setRecognizingCallback(ai_engine::lm::speech::RecognitionResultCallback callback) override;
    void setRecognizedCallback(ai_engine::lm::speech::RecognitionResultCallback callback) override;

    void setSynthesizingCallback(ai_engine::lm::speech::SynthesisResultCallback callback) override;
    void setSynthesizedCallback(ai_engine::lm::speech::FinishedCallback callback) override;

    // 语音识别
    bool initContinuousRecognitionModule(ai_engine::lm::EngineError &error) override;
    bool startContinuousRecognition(const std::string &params, ai_engine::lm::EngineError &error) override;
    bool writeContinuousRecognitionAudioData(const std::vector<char> &audioData, ai_engine::lm::EngineError &error) override;
    bool stopContinuousRecognition(ai_engine::lm::EngineError &error) override;
    bool destroyContinuousRecognitionModule(ai_engine::lm::EngineError &error) override;

    bool initRecognizeOnceModule(ai_engine::lm::EngineError &error) override;
    bool recognizeOnce(const std::string &params, const std::vector<char> &audioData, ai_engine::lm::EngineError &error) override;
    bool destroyRecognizeOnceModule(ai_engine::lm::EngineError &error) override;

    // 语音合成
    bool initContinuousSynthesisModule(ai_engine::lm::EngineError &error) override;
    bool startContinuousSynthesis(const std::string &params, ai_engine::lm::EngineError &error) override;
    bool writeContinuousSynthesisText(const std::string &text, ai_engine::lm::EngineError &error) override;
    bool stopContinuousSynthesis(ai_engine::lm::EngineError &error) override;
    bool destroyContinuousSynthesisModule(ai_engine::lm::EngineError &error) override;

    bool initSynthesizeOnceModule(ai_engine::lm::EngineError &error) override;
    bool synthesizeOnce(const std::string &params, const std::string &text, ai_engine::lm::EngineError &error) override;
    bool destroySynthesizeOnceModule(ai_engine::lm::EngineError &error) override;

private:
    enum class OperationType {
        RecognizeOnce,
        ContinuousRecognition,
        SynthesizeOnce,
        ContinuousSynthesis
    };

    const char* asrHostUrl_ = "wss://iat-api.xfyun.cn/v2/iat";
    const char* realtimeAsrHostUrl_ = "wss://rtasr.xfyun.cn/v1/ws";
    const char* ttsHostUrl_ = "wss://tts-api.xfyun.cn/v2/tts";

    std::string apiKey_;
    std::string secretKey_;
    std::string apiKeyRealTime_;
    std::string apiKeyContinuous_;
    std::string secretKeyContinuous_;
    std::string appId_;

    ai_engine::lm::speech::RecognitionResultCallback recognizingCallback_;
    ai_engine::lm::speech::RecognitionResultCallback recognizedCallback_;
    ai_engine::lm::speech::SynthesisResultCallback synthesizingCallback_;
    ai_engine::lm::speech::FinishedCallback synthesizedCallback_;

    bool doSendData(const char *data, int dataLength, int type);

    std::string format_;
    int rate_{0};
    int channel_{0};
    int currentRole_{0};
    void setRecognitionParams_(const std::string &params);
    bool doReceiveOnceRecognitionData();
    friend bool doReceiveContinuousRecognitionData(XunfeiSpeechEnginePrivate* self);

    int speed_{0};
    int volume_{0};
    int pitch_{0};
    Json::Value lastRecognitionResult_;
    OperationType currentOperation_;
    void setSynthesisParams_(const std::string &params);
    friend bool doReceiveSynthesisData(XunfeiSpeechEnginePrivate* self);

    const int maxWebDelay_{1000 * 10};
    int currentWebDelay_{0};
    bool isPinging_{false};
    bool connectWebsocket(std::string hostUrl);
    bool connectRealtimeAsrWebsocket();
    friend void onWebSocket(bool (*doReceiveData)(XunfeiSpeechEnginePrivate* self), XunfeiSpeechEnginePrivate* self);
    bool testWebSocket();
    void closeWebsocket();

    CURL *currentCurl_{nullptr};
    std::thread *currentThread_{nullptr};
    ai_engine::lm::EngineError currentError_;
    bool currentStatus();
    void runCallbackWithError(const ai_engine::lm::EngineError& error, OperationType type);

    void releaseResources();

    bool isStopped_{false};

    std::string incompleteResult_;
};

#endif
